//////////////////////////////////////////////////////////////////////
// auto.cpp - Auto Skills by Maulei (jonathanwu@hongkong.com)
// -------------------------------------------------------------------
// A D2HackIt module that casts spells and flashes auras automatically
// for a player.
//////////////////////////////////////////////////////////////////////
#include "External/D2Hackit.h"
#include "External/D2Client.h"
#include <time.h>

//////////////////////////////////////////////////////////////////////
// Version History
//
// Ver.  Date       Details
// -------------------------------------------------------------------
// 0.10  22-Nov-01  Initial code
// 0.20  29-Nov-01  Skill information are now global variables
// 0.30  29-Nov-01  Add debug message
// 0.40  30-Nov-01  Fix the timer bug.  Support few more skills
// 0.50  04-Dec-01  Aura Flashing, more skills, statistics
// 0.55  12-Dec-01  Support INI read / write
// 0.56  12-Dec-01  Weapon swap during auto-cast
// 0.60  13-Dec-01  Auto cast upon spell expiry
// 0.61  14-Dec-01  Support Meditation, Blessed Aim, Cleansing
// 0.62  24-Dec-01  Enhance Aura Flash to support player override
//                  Support 'reset' command
// 0.63  25-Dec-01  Fixed a nasty bug that cannot join a game properly
// 0.70  31-Dec-01  Fixed memory leak problem
//                  Rewrite skill recast status byte checking logic
//                  Support Inner Sight, Slow Missiles for Amazon
// 0.71  02-Jan-02  Add a routine to handle missing INI file
//                  Fixed memory leak problem for reading INI file
// 0.72  04-Jan-02  Log file logic is implemented
//                  Fixed a bug for auto recast upon spell expiry.
// 0.73  13-Jan-02  Rearrange spell orders. 'Pet spells' are at the
//                  end of list
//                  New Command 'pet' to list the pet spells
//                  Capture Skill Level to prevent casting invalid
//                  spells
// 0.75  29-Jan-02  Add a checking to prevent auto-casting spell during
//                  ZEALing / WWing
//                  Player can force NOT to auto cast when spell effect
//                  expired.
//                  Fixed a bug that Auto Skills does not work with
//                  Assassin skills
//                  Revise in/out packet capture routine
// 0.76  29-Jan-02  Remove the overhead message
// 0.77  30-Jan-02  Recast can be resumed after casting a spell
// 0.80  31-Jan-02  Fixed a bug that wrongly capture the skill of
//                  another player.  This causes disconnection!
//                  Skill level checking for Paladin's Aura added
//                  Mana save implemented
// 0.81  06-Feb-02  Fixed D2HackIt v0.56 incompatability problem
//                  Fixed missing INI crashing problem *again, sigh*
//                  Recast Reminder Message is added.
// 0.82  10-Feb-02  Recast Reminder Message can be disabled by 
//                  '.set verbose off'
//////////////////////////////////////////////////////////////////////

//////////////////////////////////////////////////////////////////////
// CLIENTINFO
//////////////////////////////////////////////////////////////////////
DWORD   ModuleVersion=MAKELONG(0, 82);
char    ModuleAuthor[]="Maulei";
char    ModuleWebsite[]="jonathan.makes.it";
char    ModuleDescription[]="Auto Skills - casts spells and flashes auras automatically";
char    ModuleEmail[]="jonathanwu@hongkong.com";

//////////////////////////////////////////////////////////////////////
// Function Definition
//////////////////////////////////////////////////////////////////////
DWORD WINAPI ThreadProc(LPVOID lpParameter);
BOOL PRIVATE OnGameCommandHelp(char** argv, int argc);
BOOL PRIVATE OnGameCommandAura(char** argv, int argc);
BOOL PRIVATE OnGameCommandCast(char** argv, int argc);
BOOL PRIVATE OnGameCommandStart(char** argv, int argc);
BOOL PRIVATE OnGameCommandStop(char** argv, int argc);
BOOL PRIVATE OnGameCommandStat(char** argv, int argc);
BOOL PRIVATE OnGameCommandAuraTime(char** argv, int argc);
BOOL PRIVATE OnGameCommandManaSave(char** argv, int argc);
BOOL PRIVATE OnGameCommandSwap(char** argv, int argc);
BOOL PRIVATE OnGameCommandReset(char** argv, int argc);
BOOL PRIVATE OnGameCommandLog(char** argv, int argc);
BOOL PRIVATE OnGameCommandPet(char** argv, int argc);

void SetSkill(char** argv, int argc);
void SetPetSpell(char** argv, int argc);
void ShowSkills();
void ShowAuraQueue();
void InitAuraQueue();
void ShowManaSave();
void ShowAuraFlashTime();
void ShowAura();
void SetAura(char** argv, int argc);
BOOL IsAuraPacket(BYTE);
void AuraFlash();
void SwapWeapon();
void LoadConfig();
BOOL IsInTown();
void OverheadMsg(char *);
void WriteLog(char *, char *);
void ShowPetSpells();
BOOL LogInPacket(BYTE);
BOOL LogOutPacket(BYTE);

//////////////////////////////////////////////////////////////////////
// ClientCore.cpp stuff
//////////////////////////////////////////////////////////////////////
char ConfigPath[1024];
FUNCTIONENTRYPOINTS	*server;

//////////////////////////////////////////////////////////////////////
// Class Definition
//////////////////////////////////////////////////////////////////////
enum {
	Amazon = 0,	Sorceress, Necromancer, Paladin, Barbarian, Druid, Assassin, TotalClass
};

char *ClassName[TotalClass] = {
    "Amazon", "Sorceress", "Necromancer", "Paladin", "Barbarian", "Druid", "Assassin"
};

//////////////////////////////////////////////////////////////////////
// Skills Definition
//////////////////////////////////////////////////////////////////////
enum {
	InnerSight=0, SlowMissiles, Valkyrie,
	ThunderStorm, EnergyShield, ChillingArmor, FrozenArmor, ShiverArmor, Enchant,
	BoneArmor, PoisonNova, ClayGolem, BloodGolem, FireGolem,
	HolyShield,
	BattleCommand, BattleOrders, BattleCry, Shout,
	WereWolf, WereBear, CycloneArmor, OakSage, HeartOfWolverine, SpiritOfBarbs, SummonSpiritWolf, SummonDireWolf, SummonGrizzly, Raven, PoisonCreeper, CarrionVine, SolarCreeper,
	BurstOfSpeed, Fade, BladeShield, Venom, ShadowWarrior, ShadowMaster,
	TotalSkills
};

int SkillNumber[TotalSkills] = {
    8, 17, 32,
    57, 58, 60, 40, 50, 52,
    68, 92, 75, 85, 94,
    117,
    155, 149, 146, 138,
    223, 228, 235, 226, 236, 246, 227, 237, 247, 221, 222, 231, 241,
    258, 267, 277, 278, 268, 279
};

char *SkillName[TotalSkills] = {
    "Inner Sight", "Slow Missiles", "Valkyrie",
    "Thunder Storm", "Energy Shield", "Chilling Armor", "Frozen Armor", "Shiver Armor", "Enchant",
    "Bone Armor", "Poison Nova", "Clay Golem", "Blood Golem", "Fire Golem",
    "Holy Shield",
    "Battle Command", "Battle Orders", "Battle Cry", "Shout",
    "Werewolf", "Werebear", "Cyclone Armor", "Oak Sage", "Heart of Wolverine", "Spirit of Barbs", "Summon Spirit Wolf", "Summon Dire Wolf", "Summon Grizzly", "Raven", "Poison Creeper", "Carrion Vine", "Solar Creeper",
    "Burst of Speed", "Fade", "Blade Shield", "Venom", "Shadow Warrior", "Shadow Master"
};

//////////////////////////////////////////////////////////////////////
// Packet 0x12 status byte mapping
// Status byte offset (0 is start of packet, not start of status bytes)
// Status byte value, 0 means no status byte checking
//////////////////////////////////////////////////////////////////////
char StatusByte[TotalSkills] = {
	8, 16, 0,
	10, 9, 8, 7, 17, 8,
	7, 0, 0, 0, 0,
	18,
	12, 10, 17, 9,
	23, 23, 24, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,
	25, 25, 25, 0, 0, 0
};

unsigned char StatusBit[TotalSkills] = {
	0x02, 0x80, 0x20,
	0x40, 0x40, 0x10, 0x04, 0x01, 0x01,
	0x40, 0x00, 0x00, 0x00, 0x00,
	0x20,
	0x08, 0x01, 0x02, 0x04,
	0x08, 0x10, 0x80, 0x20, 0x10, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
	0x20, 0x80, 0x40, 0x00, 0x80, 0x80
};

// Pet Spells
int PetSpells[TotalClass]   = {1, 0, 3, 0, 0, 10, 2};
int PetOffset[TotalClass]   = {2, 0, 2, 0, 0, 3, 4};

// Number of skills supported for each class
int NumSkills[TotalClass]   = {3, 6, 5,  1,  4, 13, 6};

// Skill offset for each class in 'skill array'
int SkillOffset[TotalClass] = {0, 3, 9, 14, 15, 19, 32};

DWORD Interval[TotalSkills], Elapsed[TotalSkills], Casted[TotalSkills];
BOOL Recast[TotalSkills], RecastOnExpiry[TotalSkills], RecastMsg[TotalSkills];
time_t LastActive[TotalSkills];

// Skill Level Table
const int MaxSkill = 318;
BYTE SkillLevel[MaxSkill];

//////////////////////////////////////////////////////////////////////
// Auras Definition
//////////////////////////////////////////////////////////////////////
int NumAura = 11;
char *AuraName[12] = {
    "c1AURA FLASH OFFc0",
	"Might", "Thorns", "Blessed Aim", "Defiance", "Concentration", "Holy Freeze",
	"Conviction", "Salvation", "Fanaticism", "Cleansing", "Meditation"
};
int AuraNumber[12] = {
    0, 98, 103, 108, 104, 113, 114, 123, 125, 122, 109, 120
};
int AuraQueue[12];
int AuraInterval, AuraElapsed, Flashed=0, CurrentAura=0;
time_t AuraLastActive;

time_t Current;
double temp;
int Swap=0;
int CurrentHP, CurrentMP, CurrentLvl;
int ManaSave=0;
BOOL FirstGame=TRUE, FirstPacket59=TRUE;
BYTE PlayerID, CharacterClass;
//BOOL SpellCastable=FALSE;


// Log Related
FILE *logfile;
int LogFile=0;
char LogTag[11], LogMsg[1024];

// For writing INI file
char _Character[64], _Key[64], _Value[64];
char Msg1[256];

typedef struct thread_data_t
{
	// Thread ID
	HANDLE   Thread;

	// TRUE if in a game
	BOOL  Active;

    // For getting info :)
    THISGAMESTRUCT*   thisgame;

    // Saved old skill
    int  OldSkill;

} THREAD_DATA;

THREAD_DATA td;

//////////////////////////////////////////////////////////////////////
// Module Commands
//////////////////////////////////////////////////////////////////////

MODULECOMMANDSTRUCT ModuleCommands[]=
{
	{
		"help",  OnGameCommandHelp,
		"help c0 List commands available in this module.\n"
		"<command> help c0 Shows detailed help for <command> in this module."
	},
	{
		"aura",  OnGameCommandAura,
		"aura c0 Display or select up to 11 auras for flashing\n"
	},
	{
		"at",    OnGameCommandAuraTime,
		"at c0 Display or specify aura flash time interval (in sec)\n"
	},
	{
		"cast",  OnGameCommandCast,
		"cast c0 Display or specify auto cast skill and recast time interval (in sec)\n"
	},
	{
		"pet",   OnGameCommandPet,
		"pet c0 Display or specify auto cast skill and recast time interval (in sec)\n"
	},
	{
		"mana",  OnGameCommandManaSave,
		"cast c0 Display or specify amount of mana save before auto cast\n"
	},
	{
		"start", OnGameCommandStart,
		"start c0 Start or reset Auto Skills\n"
	},
	{
		"stop",  OnGameCommandStop,
		"stop c0 Temporarily stops Auto Skills\n"
	},
	{
		"stat",  OnGameCommandStat,
		"stat c0 Display Auto Skills statistics\n"
	},
	{
		"swap",  OnGameCommandSwap,
		"swap c0 Choose to swap weapon when auto cast or not.\n"
	},
	{
		"reset", OnGameCommandReset,
		"reset c0 Reset auto cast, aura flash setting for this character.\n"
	},
	{
		"log",   OnGameCommandLog,
		"log c0 Display or specify Auto Skills to log its activity for debug purpose\n"
	},
	{NULL}
};

//////////////////////////////////////////////////////////////////////
// OnClientStart
//////////////////////////////////////////////////////////////////////
BOOL EXPORT OnClientStart()
{
	FILE *file;

	// Make sure ini file exists
	sprintf(Msg1, "%s\\%s", ConfigPath, "auto.ini");
	file = fopen(Msg1, "a+");
	fclose(file);

	// Initialize statistics, last active time
    for (int i=0; i<TotalSkills; i++)
    {
		Casted[i] = 0;
	    Elapsed[i] = 0;
        time(&LastActive[i]);
    }
    Flashed = 0;
    InitAuraQueue();
    td.OldSkill = 0;
    return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnGameJoin
//////////////////////////////////////////////////////////////////////
VOID EXPORT OnGameJoin(THISGAMESTRUCT* thisgame)
{
	int i, j;

	td.thisgame=thisgame;

	// Setup Auto Cast & Aura Flash settings from INI file
	LoadConfig();

	sprintf(LogMsg, "%s %s [ID:%.2X] joined a game.", ClassName[td.thisgame->player->CharacterClass], td.thisgame->player->PlayerName, td.thisgame->player->PlayerID);
	sprintf(LogTag, "JoinGame  ");
	WriteLog(LogTag, LogMsg);

	// Status Report
	ShowSkills();
	ShowManaSave();
    ShowAuraQueue();

    // Recast spell immediately when joining a new game.
    j = SkillOffset[td.thisgame->player->CharacterClass];
    for (i=0; i<NumSkills[td.thisgame->player->CharacterClass]; i++)
	{
		Recast[i+j] = TRUE;
		RecastMsg[i+j] = FALSE;
		time(&LastActive[i+j]);
	}
	
	// Start thread
	td.Active=TRUE;
	DWORD dummy=0;
    td.Thread = CreateThread(NULL,0,ThreadProc,&td,0,&dummy);
}

//////////////////////////////////////////////////////////////////////
// OnGameLeave
//////////////////////////////////////////////////////////////////////
VOID EXPORT OnGameLeave(THISGAMESTRUCT* thisgame)
{
	// Set state to inactive, the thread will terminate itself
    td.Active=FALSE;
	FirstPacket59=TRUE;

    // Wait for thread to die
    if (td.Thread)
    {
		server->GamePrintVerbose("Auto: Waiting max 2 seconds for worker thread to exit.");
        WaitForSingleObject(td.Thread, 2000);
        server->GamePrintVerbose("Auto: OK!");
    }
}

//////////////////////////////////////////////////////////////////////
// OnGameCommandAura
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandAura(char** argv, int argc)
{
	// Command for paladin only
    if (td.thisgame->player->CharacterClass != CharClassPaladin)
	{
		sprintf(Msg1, "Auto: c1Command not available! c3%s is not a Paladin!c0", td.thisgame->player->PlayerName);
		server->GamePrintError(Msg1);
		return TRUE;
	}

	if (argc==2)
    {
		ShowAura();
		ShowAuraQueue();
    }
    else if (argc > 2 && argc <= NumAura+2)
        SetAura(argv, argc);
    else
    {
		sprintf(Msg1, "c1Syntax:c0 .auto aura <aura number> <aura number> ...");
        server->GamePrintInfo(Msg1);
	}
    return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnGameCommandAuraTime
// 1. Update Aura Flash Interval
// 2. Write Aura Flash Interval to INI file
// 3. Display message for command syntax error
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandAuraTime(char** argv, int argc)
{
	// Command for paladin only
    if (td.thisgame->player->CharacterClass != CharClassPaladin)
	{
		sprintf(Msg1, "Auto: c1Command not available! c3%s is not a Paladin!c0", td.thisgame->player->PlayerName);
		server->GamePrintError(Msg1);
		return TRUE;
	}

    if (argc==2)
	    ShowAuraFlashTime();
    else if (argc==3)
    {
        AuraInterval = abs(atoi(argv[2]));
 	    ShowAuraFlashTime();
		sprintf(_Character, "%s", td.thisgame->player->PlayerName);
		sprintf(_Key, "%s", "Aura Flash Interval");
		sprintf(_Value, "%d", AuraInterval);
		server->SetHackProfileString("auto", _Character, _Key, _Value);
    }
    else
    {
		sprintf(Msg1, "c1Syntax:c0 .auto auratime <interval>");
		server->GamePrintInfo(Msg1);
    }
    return TRUE ;
}

//////////////////////////////////////////////////////////////////////
// OnGameCommandManaSave
// 1. Update Mana Save
// 2. Write Mana Save to INI file
// 3. Display message for command syntax error
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandManaSave(char** argv, int argc)
{
    if (argc==2)
    {
	    ShowManaSave();
	}
    else if (argc==3)
    {
        ManaSave = abs(atoi(argv[2]));
 	    ShowManaSave();
		sprintf(_Character, "%s", td.thisgame->player->PlayerName);
		sprintf(_Key, "%s", "Mana Save");
		sprintf(_Value, "%d", ManaSave);
		server->SetHackProfileString("auto", _Character, _Key, _Value);
    }
    else
    {
		sprintf(Msg1, "c1Syntax:c0 .auto mana <mana>");
		server->GamePrintInfo(Msg1);
    }
    return TRUE ;
}

//////////////////////////////////////////////////////////////////////
// OnGameCommandCast
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandCast(char** argv, int argc)
{
    if (argc==2)
	{
        ShowSkills();
        ShowAuraQueue();
    }
    else if (argc==4)
    {
        SetSkill(argv, argc);
	}
    else
	{
        sprintf(Msg1, "c1Syntax:c0 .auto cast <skill number> <interval / x>");
        server->GamePrintInfo(Msg1);
    }
	return TRUE ;
}

//////////////////////////////////////////////////////////////////////
// OnGameCommandStart
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandStart(char** argv, int argc)
{
    if (argc!=2)
		return FALSE;

    OnGameLeave(0);
    OnGameJoin(td.thisgame);
    return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnGameCommandStop
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandStop(char** argv, int argc)
{
    OnGameLeave(0);
    return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnGameCommandSwap
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandSwap(char** argv, int argc)
{
    int i;

    if (argc==3)
    {
		Swap = 0;
        i = atoi(argv[2]);
        if (i) Swap = 1;
		sprintf(_Character, "%s", td.thisgame->player->PlayerName);
		sprintf(_Key, "%s", "Swap Weapon");
		sprintf(_Value, "%d", Swap);
		server->SetHackProfileString("auto", _Character, _Key, _Value);
    }

    sprintf(Msg1, "Swap weapon when auto cast is now c1%sc0", Swap?"On":"Off");
    server->GamePrintInfo(Msg1);

    return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnGameCommandLog
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandLog(char** argv, int argc)
{
    int i;

    if (argc==3)
    {
		LogFile = 0;
        i = atoi(argv[2]);
        if (i) LogFile = 1;
		sprintf(_Character, "%s", td.thisgame->player->PlayerName);
		sprintf(_Key, "%s", "Log");
		sprintf(_Value, "%d", LogFile);
		server->SetHackProfileString("auto", _Character, _Key, _Value);
    }

    sprintf(Msg1, "Debug log is now c1%sc0", LogFile?"On":"Off");
    server->GamePrintInfo(Msg1);

    return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnGameCommandReset
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandReset(char** argv, int argc)
{
	// Reset Auto Cast settings
	for (int i=0; i<NumSkills[td.thisgame->player->CharacterClass]; i++)
	{
		int targetSkill = i + SkillOffset[td.thisgame->player->CharacterClass];
		sprintf(_Character, "%s", td.thisgame->player->PlayerName);
		sprintf(_Key, "%s", SkillName[targetSkill]);
		sprintf(_Value, "%d", 0);
		server->SetHackProfileString("auto", _Character, _Key, _Value);
		Interval[targetSkill] = 0;
		RecastOnExpiry[targetSkill] = FALSE;
	}
    sprintf(Msg1, "Auto cast settings for c2%sc0 reset.", _Character);
    server->GamePrintInfo(Msg1);

	// Reset Aura Flash settings
	if (td.thisgame->player->CharacterClass == CharClassPaladin)
	{
		for (i=0; i<NumAura; i++)
		{
			sprintf(_Character, "%s", td.thisgame->player->PlayerName);
			sprintf(_Key, "Aura Flash %d", i+1);
			sprintf(_Value, "%d", AuraQueue[i]);
			server->SetHackProfileString("auto", _Character, _Key, _Value);
			AuraQueue[i] = 0;
		}
	    sprintf(Msg1, "Aura flash settings for c1%sc0 reset.", _Character);
		server->GamePrintInfo(Msg1);
	}

	return TRUE;
}

//////////////////////////////////////////////////////////////////////
// OnGameCommandPet
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandPet(char** argv, int argc)
{
    if (argc==2)
	{
        ShowPetSpells();
        return TRUE ;
    }
    else if (argc==4)
    {
        SetPetSpell(argv, argc);
        return TRUE;
	}
    else
	{
        sprintf(Msg1, "c1Syntax:c0 .auto pet <skill number> <interval>");
        server->GamePrintInfo(Msg1);
	    return TRUE;
    }
}

//////////////////////////////////////////////////////////////////////
// OnGamePacketBeforeReceived
// 1.  Capture current skills
// 2.  Capture unit status flag for auto cast upon skill expiry
// 3.  Capture Skill Level
// 4.  Capture HP & MP
//////////////////////////////////////////////////////////////////////
DWORD EXPORT OnGamePacketBeforeReceived(BYTE* aPacket, DWORD aLen)
{
	int i, j;

	// Select packets to Log
	if (LogInPacket(aPacket[0]))
	{
		sprintf(LogMsg, "");
		for(i=0; i<aLen; i++)
			sprintf(LogMsg, "%s %.2X", LogMsg, aPacket[i]);
		switch (aPacket[0])
		{
			case 0x12: sprintf(LogTag, "R:UnitStus"); break;
			case 0x23: sprintf(LogTag, "R:OSkill%.2X", aPacket[7]); break;
			case 0x56: sprintf(LogTag, "R:UntInf%.2X", aPacket[1]); break;
			case 0x59: sprintf(LogTag, "R:PlyrName"); break;
			case 0x94: sprintf(LogTag, "R:SkillInf"); break;
			default:   sprintf(LogTag, "R:Packet%.2X", aPacket[0]); break;
		}
		WriteLog(LogTag, LogMsg+1);
	}

	// Capture Player ID
	if (aPacket[0] == 0x59)
	{
		if (FirstPacket59)
		{
			PlayerID = aPacket[1];
			CharacterClass = aPacket[5];
			sprintf(LogTag, "FirstPkt59");
			sprintf(LogMsg, "Player ID:%.2X Char Class:%.2X", PlayerID, CharacterClass);
			WriteLog(LogTag, LogMsg);
			FirstPacket59 = FALSE;
		}
		return aLen;
	}

	// Capture old skills
	if (aPacket[0] == 0x23 && aPacket[1] == 0x00 && aPacket[2] == PlayerID && aPacket[6] == 0x00)
	{
		td.OldSkill = aPacket[7] + (aPacket[8] << 8);
		return aLen;
	}

	// Capture Skill Packet
	if (aPacket[0] == 0x94)
	{
		FirstGame = FALSE;
		for (i=0; i<MaxSkill; i++)
			SkillLevel[i] = 0;

		for (i=0; i<aPacket[1]; i++)
		{
			j = *((WORD *)(aPacket+(i*3+6)));
			SkillLevel[j] = aPacket[i*3+8];
		}
		return aLen;
	}

	// Update HP, MP
	if (aPacket[0] == 0x95)
	{
		CurrentHP = *((WORD *)(aPacket + 1)) & 0x0FFF;
		CurrentMP = (*((WORD *)(aPacket + 3)) & 0x0FFF) << 1;
	}

	// Update item status packet
	if (aPacket[0] == 0x12 && aPacket[1] == 0x00 && aPacket[2] == PlayerID)
	{
		// Dump Packet
		sprintf(LogMsg, "");
		for(i=0; i<aLen; i++)
			sprintf(LogMsg, "%s --", LogMsg);
		sprintf(LogTag, "UnitStatus");

		// Analyse Packet
		sprintf(Msg1, "");
		j = SkillOffset[td.thisgame->player->CharacterClass];
		for (i=0; i<NumSkills[td.thisgame->player->CharacterClass]; i++)
		{
			if (StatusByte[i+j])
			{
				if (!(aPacket[StatusByte[i+j]] & StatusBit[i+j]))
				{
					if (Interval[i+j] || RecastOnExpiry[i+j])
					{
						if (StatusBit[i+j] > 0x0F)
							LogMsg[StatusByte[i+j]*3+1] = '*';
						else
							LogMsg[StatusByte[i+j]*3+2] = '*';

						sprintf(Msg1, "%s [%s:%s]", Msg1, SkillName[i+j], "Off");
					}
					if  (RecastOnExpiry[i+j])
						Recast[i+j] = TRUE;
				}
				else
				{
					sprintf(Msg1, "%s [%s:%s]", Msg1, SkillName[i+j], "On");
				}
			}
		}
		sprintf(LogTag, "UnitStatus");
		WriteLog(LogTag, LogMsg+1);
		sprintf(LogTag, "UnitStatus");
		WriteLog(LogTag, Msg1+1);

		return aLen;
	}
    return aLen;
}

//////////////////////////////////////////////////////////////////////
// OnGamePacketBeforeSent
// 1.  Update Aura last active time if player choose an aura
// 2.  Setup spell castable flag
//////////////////////////////////////////////////////////////////////
DWORD EXPORT OnGamePacketBeforeSent(BYTE* aPacket, DWORD aLen)
{
	int i;

	// Select packets to Log
	if (LogOutPacket(aPacket[0]))
	{
		sprintf(LogMsg, "");
		for(i=0; i<aLen; i++)
			sprintf(LogMsg, "%s %.2X", LogMsg, aPacket[i]);
		switch (aPacket[0])
		{
			case 0x0c: sprintf(LogTag, "S:UseSkill"); break;
			case 0x08: sprintf(LogTag, "S:HoldLeft"); break;
			case 0x09: sprintf(LogTag, "S:HoldLeft"); break;
			case 0x0f: sprintf(LogTag, "S:HoldRght"); break;
			case 0x10: sprintf(LogTag, "S:HoldRght"); break;
			case 0x3c: sprintf(LogTag, "S:CSkill%.2X", aPacket[1]); break;
			case 0x66: sprintf(LogTag, "S:QuitGame"); break;
			default:   sprintf(LogTag, "S:Packet%.2X", aPacket[0]); break;
		}
		WriteLog(LogTag, LogMsg+1);
	}

	/*
	// Character walking / running / casting
	if ((aPacket[0] >= 0x01 && aPacket[0] <= 0x06) || aPacket[0] == 0x0c || aPacket[0] == 0x0d)
	{
		if (!SpellCastable)
		{
			SpellCastable = TRUE;
			//OverheadMsg("Spell castable!");
		}
	}

	// Mouse button is holding down for a skill
	if (aPacket[0] == 0x08 || aPacket[0] == 0x09 || aPacket[0] == 0x0F || aPacket[0] == 0x10)
	{
		if (SpellCastable)
		{
			SpellCastable = FALSE;
			//OverheadMsg("Spell NOT castable!");
		}
	}
	*/

	// Aura Packet?  If yes, update Aura Last Active Time
	if (aPacket[0] == 0x3c && AuraInterval != 0)
	{
		if (IsAuraPacket(aPacket[1])) time(&AuraLastActive);
	}

	return aLen;
}

//////////////////////////////////////////////////////////////////////
// OnGameCommandStat
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandStat(char** argv, int argc)
{
    int j = SkillOffset[td.thisgame->player->CharacterClass];

    // Loop through Skills (for a particular class)
    sprintf(Msg1, "Auto Skills statistics for %s", ClassName[td.thisgame->player->CharacterClass]);
    server->GamePrintInfo(Msg1);
    for (int i=0; i<NumSkills[td.thisgame->player->CharacterClass]; i++)
    {
        sprintf(Msg1, "c0%d. c4%sc0: c2%dc0 times", i, SkillName[i+j], Casted[i+j]);
        server->GamePrintInfo(Msg1);
	}

    if (td.thisgame->player->CharacterClass == CharClassPaladin)
    {
        sprintf(Msg1, "c0Aura flash c4%dc0 times", Flashed);
        server->GamePrintInfo(Msg1);
    }
	//sprintf(Msg1, "HP:%d MP:%d", CurrentHP, CurrentMP);
	//OverheadMsg(Msg1);
    return TRUE;
}

//////////////////////////////////////////////////////////////////////
// ThreadProc
//////////////////////////////////////////////////////////////////////
DWORD WINAPI ThreadProc(LPVOID lpParameter)
{
    THREAD_DATA *td=(THREAD_DATA*)lpParameter;

    while (td->Active)
    {
		// Use In game Player ID and character class
		CharacterClass = td->thisgame->player->CharacterClass;
		PlayerID = td->thisgame->player->PlayerID;

		// Aura Flash for Paladin
        if (td->thisgame->player->CharacterClass == CharClassPaladin)
			AuraFlash();

		// Loop through Skills (for a particular class)
 	    int j = SkillOffset[td->thisgame->player->CharacterClass];
        for (int i=0; i<NumSkills[td->thisgame->player->CharacterClass]; i++)
        {
			// If spell will be recasted in 3 seconds
			// Display a message to remind user to give time Auto Skills to cast spell
			if (Elapsed[i+j] >= Interval[i+j] - 4)
			{
				if (!RecastMsg[i+j])
				{
					sprintf(Msg1, "Auto: c4%sc0 will be casted in c13c0 seconds", SkillName[i+j]);
					server->GamePrintVerbose(Msg1);
				}
				RecastMsg[i+j] = TRUE;
			}

			// Spell is castable?
			// Recast Interval on -OR- Recast on expiry on?
			// Non zero Skill level -OR- First Game?
            //if (SpellCastable && (Interval[i+j] || RecastOnExpiry[i+j]) && (SkillLevel[SkillNumber[i+j]] || FirstGame))
            if ((Interval[i+j] || RecastOnExpiry[i+j]) && (SkillLevel[SkillNumber[i+j]] || FirstGame))
			{
			    time(&Current);
	            temp = difftime(Current, LastActive[i+j]);
		        Elapsed[i+j] = temp;

				// Elapsed time reach recast interval?
				// Recast on Expiry for that spell?
				// Mana > Mana Save?
 			    if (((Elapsed[i+j] > Interval[i+j] - 1) || Recast[i+j]) && CurrentMP > ManaSave)
				{
					if  (!IsInTown())
					{
						// Swap weapon before auto cast?
					    if (Swap) SwapWeapon();

					    // Build packets to send
					    BYTE* b=new BYTE[20];

				 	    sprintf(Msg1, "Auto: c4%sc0 casted", SkillName[i+j]);
					    server->GamePrintVerbose(Msg1);

						sprintf((char*)b, "%c%c%c%c%c%c%c%c%c", 0x3c, SkillNumber[i+j]&0xFF,(SkillNumber[i+j]&0xFF00)>>8,0,0,0xff,0xff,0xff,0xff);
					    server->GameSendPacketToServer(b, 9);

					    // Cast skill on your current location
					    sprintf((char*)b, "%c%c%c%c%c",
                            0x0c,
                            LOBYTE(td->thisgame->player->PlayerPositionX),
                            HIBYTE(td->thisgame->player->PlayerPositionX),
                            LOBYTE(td->thisgame->player->PlayerPositionY),
                            HIBYTE(td->thisgame->player->PlayerPositionY)
					    );
					    server->GameSendPacketToServer(b, 5);
					    Casted[i+j]++;
					    Recast[i+j] = FALSE;

						// Reset timer
					    time(&Current);
					    time(&LastActive[i+j]);
					    temp = difftime(Current, LastActive[i+j]);
					    Elapsed[i+j] = temp;

					    // Change skill back
					    if (Swap)
						{
							Sleep(400);
							SwapWeapon();
						}

						if (td->OldSkill && !Swap)
						{
							sprintf((char*)b, "%c%c%c%c%c%c%c%c%c", 0x3c, td->OldSkill&0xFF,(td->OldSkill&0xFF00)>>8,0,0,0xff,0xff,0xff,0xff);
						    server->GameSendPacketToServer(b, 9);
							Sleep(150);
						}

					    // Kill
					    delete b;
						
						// Reset remind message flags
						RecastMsg[i+j] = FALSE;
					}
				}
				else
				{
					time(&Current);
				    temp = difftime(Current, LastActive[i+j]);
				    Elapsed[i+j] = temp;
				}
			}
			Sleep(250);

			if (!td->Active)
			{
				td->Thread=0;
				return 0;
			}
		}
	}

	td->Thread=0;
	return 0;
}

//////////////////////////////////////////////////////////////////////
// SetSkill
// 1. Setup auto cast skills
// 2. Update auto cast skills to INI file
// 3. Display error message if skill number is out of range
//////////////////////////////////////////////////////////////////////
void SetSkill(char **argv, int argc)
{
	int i, targetSkill, targetLevel;
	char *lvl, *ptr, c='x';

	i = atoi(argv[2]);

	// Within range of supported skills?
    if ((i>=0) && (i < NumSkills[td.thisgame->player->CharacterClass]))
	{
		targetSkill = i + SkillOffset[td.thisgame->player->CharacterClass];

		// Setup keys to update INI file
		sprintf(_Character, "%s", td.thisgame->player->PlayerName);
		sprintf(_Key, "%s", SkillName[targetSkill]);

		// Set on 'Recast On Expiry' mode?
		lvl = argv[3];
		ptr = strchr(lvl, c);
		if (ptr)
		{
			// Support 'Recast on expiry' mode?
			if (StatusByte[targetSkill])
			{
				RecastOnExpiry[targetSkill] = TRUE;
				sprintf(_Value, "*");
				Interval[targetSkill] = 0;
	        	sprintf(Msg1, "c4%sc0 will be recasted when its effect expired.", SkillName[targetSkill]);
    	    	server->GamePrintInfo(Msg1);
    	    }
    	    else
    	    {
	        	sprintf(Msg1, "c4%sc0 does not support recast when expired mode.", SkillName[targetSkill]);
    	    	server->GamePrintError(Msg1);
    	    	return;
    		}
		}
		else
		{
			RecastOnExpiry[targetSkill] = FALSE;
			targetLevel = atoi(argv[3]);
			sprintf(_Value, "%d", targetLevel);
			Interval[targetSkill] = targetLevel;
	        sprintf(Msg1, "c4%sc0 will be recasted every c4%dc0 seconds.", SkillName[targetSkill], targetLevel);
    	    server->GamePrintInfo(Msg1);
		}

	    time(&Current);
	    time(&LastActive[targetSkill]);
        temp = difftime(Current, LastActive[targetSkill]);
	    Elapsed[targetSkill] = temp;

		server->SetHackProfileString("auto", _Character, _Key, _Value);
	}
	else
	{
		sprintf(Msg1, "Auto: c1Skill number error!c0 c2%dc0 is out of range!", i);
		server->GamePrintError(Msg1);
		ShowSkills();
	}
}

//////////////////////////////////////////////////////////////////////
// SetPetSpell
//////////////////////////////////////////////////////////////////////
void SetPetSpell(char **argv, int argc)
{
	int i, targetSkill, targetLevel;

	i = atoi(argv[2]);

    if ((i>=0) && (i < PetSpells[td.thisgame->player->CharacterClass]))
	{
		targetSkill = i + SkillOffset[td.thisgame->player->CharacterClass] + PetOffset[td.thisgame->player->CharacterClass];
		targetLevel = atoi(argv[3]);
		Interval[targetSkill] = targetLevel;
        sprintf(Msg1, "Type c1.auto castc0 to check all auto cast skills");
		server->GamePrintInfo(Msg1);
        sprintf(Msg1, "c4%sc0 is set to recast every c4%dc0 seconds.", SkillName[targetSkill], targetLevel);
        server->GamePrintInfo(Msg1);
	    time(&Current);
	    time(&LastActive[targetSkill]);
        temp = difftime(Current, LastActive[targetSkill]);
	    Elapsed[targetSkill] = temp;

		// Update INI file
		sprintf(_Character, "%s", td.thisgame->player->PlayerName);
		sprintf(_Key, "%s", SkillName[targetSkill]);
		sprintf(_Value, "%d", targetLevel);
		server->SetHackProfileString("auto", _Character, _Key, _Value);
	}
	else
	{
		sprintf(Msg1, "Auto: c1Pet spell number error!c0 c2%dc0 is out of range!", i);
		server->GamePrintError(Msg1);
		ShowPetSpells();
	}
}

//////////////////////////////////////////////////////////////////////
// List the supported auto cast skills of current character.
//////////////////////////////////////////////////////////////////////
void ShowSkills()
{
	int i, j, k;

	sprintf(Msg1, "%s c3%sc0 -- c8%dc0 skills supported", ClassName[td.thisgame->player->CharacterClass], td.thisgame->player->PlayerName, NumSkills[td.thisgame->player->CharacterClass]);
    server->GamePrintInfo(Msg1);
    j=SkillOffset[td.thisgame->player->CharacterClass];
    for (i=0; i<NumSkills[td.thisgame->player->CharacterClass]; i++)
    {
		if (Interval[i+j] || RecastOnExpiry[i+j])
		{
			k = Interval[i+j] - Elapsed[i+j];
			if (RecastOnExpiry[i+j])
				sprintf(Msg1, "%2d. c1%sc4%sc0 --> c7Recast when expiredc0", i, StatusByte[i+j]?"*":"", SkillName[i+j]);
			else
				sprintf(Msg1, "%2d. c1%sc4%sc0 --> c3%dc0 secs. (Next: c8%d c0secs)", i, StatusByte[i+j]?"*":"", SkillName[i+j], Interval[i+j], k > 0 ? k : 0);
		}
		else
			sprintf(Msg1, "%2d. c1%sc4%sc0 --> c1Inactivec0", i, StatusByte[i+j]?"*":"", SkillName[i+j]);
		server->GamePrintInfo(Msg1);
	}
}

void ShowPetSpells()
{
	int i, j, k;

	sprintf(Msg1, "%s c3%sc0 -- c8%dc0 pet spells supported", ClassName[td.thisgame->player->CharacterClass], td.thisgame->player->PlayerName, PetSpells[td.thisgame->player->CharacterClass]);
    server->GamePrintInfo(Msg1);

    j=SkillOffset[td.thisgame->player->CharacterClass]+PetOffset[td.thisgame->player->CharacterClass];
    for (i=0; i<PetSpells[td.thisgame->player->CharacterClass]; i++)
    {
		k = Interval[i+j] - Elapsed[i+j];
		if (Interval[i+j])
			sprintf(Msg1, "%2d. c4%sc0 --> c3%dc0 secs. (Next: c8%d c0secs.)", i, SkillName[i+j], Interval[i+j], k > 0 ? k : 0);
		else
			sprintf(Msg1, "%2d. c4%sc0 --> c1Inactivec0", i, SkillName[i+j]);
		server->GamePrintInfo(Msg1);
	}
}

void ShowManaSave()
{
    sprintf(Msg1, "Mana Save is c2%dc0", ManaSave);
    server->GamePrintInfo(Msg1);
}

void ShowAuraFlashTime()
{
    sprintf(Msg1, "Aura flashing time is c2%dc0 secs.", AuraInterval);
    server->GamePrintInfo(Msg1);
}

void ShowAuraQueue()
{
	int i;

    if (td.thisgame->player->CharacterClass != CharClassPaladin)
		return;

	sprintf(Msg1, "Aura for flashing (every %d secs)", AuraInterval);
	server->GamePrintInfo(Msg1);

	if (AuraQueue[0] == 0)
    {
		server->GamePrintInfo("c1-None-c0");
        return;
    }

    for (i=0; i<NumAura+1; i++)
    {
		if (AuraQueue[i])
		{
			sprintf(Msg1, "%2d. c4%sc0", i, AuraName[AuraQueue[i]]);
			server->GamePrintInfo(Msg1);
		}
	}
}

void InitAuraQueue()
{
   for (int i=0; i<NumAura; i++)
	   AuraQueue[i] = 0;
   CurrentAura = 0;
}

void ShowAura()
{
    int i;

	if (td.thisgame->player->CharacterClass != CharClassPaladin)
		return;

	sprintf(Msg1, "Class: c1Paladinc0 -- c3%dc0 auras supported", NumAura);
	server->GamePrintInfo(Msg1);

    for (i=0; i<NumAura+1; i++)
    {
		sprintf(Msg1, "%2d. c4%sc0", i, AuraName[i]);
        server->GamePrintInfo(Msg1);
    }
}

void SetAura(char **argv, int argc)
{
	int i, j=0, targetAura;

	InitAuraQueue();

    for (i=2; i<argc; i++)
    {
        targetAura = atoi(argv[i]);

		if (targetAura <= NumAura && targetAura > 0)
		{
	        if (SkillLevel[AuraNumber[targetAura]] || FirstGame)
	        {
	        	AuraQueue[j++] = targetAura;
            	sprintf(Msg1, "Auto: c4%sc0 selected!", AuraName[targetAura]);
	        	server->GamePrintInfo(Msg1);
	        }
	        else
	        {
	        	sprintf(Msg1, "Auto: You cannot use c4%sc0 yet", AuraName[targetAura]);
	        	server->GamePrintError(Msg1);
	        }
		}
	    else if (targetAura == 0)
		{
		    sprintf(Msg1, "Auto: Aura Flash c1disabledc0");
	        server->GamePrintInfo(Msg1);
		}
	    else
		{
            sprintf(Msg1, "Auto: c1Aura number error! c2%dc0 is out of range!  c1Aura Flash disabledc0", targetAura);
	        server->GamePrintError(Msg1);
  	        InitAuraQueue();
		}
	}

    // Update INI file
    for (i=0; i<NumAura; i++)
    {
		sprintf(_Character, "%s", td.thisgame->player->PlayerName);
		sprintf(_Key, "Aura Flash %d", i+1);
		sprintf(_Value, "%d", AuraQueue[i]);
        server->SetHackProfileString("auto", _Character, _Key, _Value);
	}

	ShowAuraQueue();
    return;
}

//////////////////////////////////////////////////////////////////////
// Check if packet 3c is an aura packet
//////////////////////////////////////////////////////////////////////
BOOL IsAuraPacket(BYTE SkillByte)
{
	char AuraList[20] = {
		99, 100, 105, 110, 104, 109, 115, 120, 124, 125,
		98, 102, 103, 108, 113, 114, 118, 119, 122, 123
	};

	for (int i=0; i<20; i++)
		if (SkillByte == AuraList[i])
			return TRUE;

	return FALSE;
}

//////////////////////////////////////////////////////////////////////
// Aura flash for paladin
//////////////////////////////////////////////////////////////////////
void AuraFlash()
{
	// return if no aura specified in active aura queue
	if (AuraQueue[CurrentAura] == 0 || AuraInterval == 0)
		return;

	// Check if interval for flashing aura is reached
	time(&Current);
    temp = difftime(Current, AuraLastActive);
    AuraElapsed = temp;

    if (AuraElapsed < AuraInterval)
		return;

    // Build packets to send
    BYTE* b=new BYTE[10];

    sprintf((char*)b, "%c%c%c%c%c%c%c%c%c", 0x3c, AuraNumber[AuraQueue[CurrentAura]],0,0,0,0xff,0xff,0xff,0xff);
    server->GameSendPacketToServer(b, 9);
    delete b;
	Flashed++;

    // Display message to notify player aura flashed.
    sprintf(Msg1, "c7%sc0 flashed", AuraName[AuraQueue[CurrentAura]]);
    server->GamePrintVerbose(Msg1);

    // Next aura in queue. Back to start of Aura Queue if end of list
    CurrentAura++;
    if (AuraQueue[CurrentAura] == 0 || CurrentAura > NumAura)
		CurrentAura = 0;
}

//////////////////////////////////////////////////////////////////////
// Send a swap weapon packet
//////////////////////////////////////////////////////////////////////
void SwapWeapon()
{
	// Build packets to send
    BYTE* b=new BYTE[1];

    sprintf((char*)b, "%c", 0x60);
    server->GameSendPacketToServer(b, 1);
    delete b;
}

//////////////////////////////////////////////////////////////////////
// Load character settings from INI file
//////////////////////////////////////////////////////////////////////
void LoadConfig()
{
	char *retval;
	char *ptr, c='*';
	int i, j;

	// Character specific Config
	sprintf(_Character, "%s", td.thisgame->player->PlayerName);

	// Loop through Skills (for a particular class)
	j = SkillOffset[td.thisgame->player->CharacterClass];
    for (i=0; i<NumSkills[td.thisgame->player->CharacterClass]; i++)
    {
		sprintf(_Key, "%s", SkillName[i+j]);
		retval = server->GetHackProfileString("auto", _Character, _Key);
		if (strlen(retval))
		{
			ptr = strchr(retval, c);
			if (ptr)
			{
				RecastOnExpiry[i+j] = TRUE;
				Interval[i+j] = 0;
			}
			else
			{
				RecastOnExpiry[i+j] = FALSE;
				Interval[i+j] = atoi(retval);
			}
			//delete retval;
		}
		else
		{
			Interval[i+j] = 0;
			sprintf(_Value, "%d", 0);
			server->SetHackProfileString("auto", _Character, _Key, _Value);
		}
	}

	// Get Swap Weapon Settings
	sprintf(_Key, "%s", "Swap Weapon");
	retval = server->GetHackProfileString("auto", _Character, _Key);

	if (strlen(retval))
	{
		Swap = atoi(retval);
		//delete retval;
	}
	else
	{
		Swap = 0;
		sprintf(_Value, "%d", 0);
		server->SetHackProfileString("auto", _Character, _Key, _Value);
	}

	// Get Mana Save
	sprintf(_Key, "%s", "Mana Save");
	retval = server->GetHackProfileString("auto", _Character, _Key);

	if (strlen(retval))
	{
		ManaSave = atoi(retval);
		//delete retval;
	}
	else
	{
		ManaSave = 0;
		sprintf(_Value, "%d", 0);
		server->SetHackProfileString("auto", _Character, _Key, _Value);
	}

	// Get Debug Log Settings
	sprintf(_Key, "%s", "Log");
	retval = server->GetHackProfileString("auto", _Character, _Key);

	if (strlen(retval))
	{
		LogFile = atoi(retval);
		//delete retval;
	}
	else
	{
		LogFile = 0;
		sprintf(_Value, "%d", 0);
		server->SetHackProfileString("auto", _Character, _Key, _Value);
	}

	AuraInterval = 0;

	// Paladin Only Settings
	if (td.thisgame->player->CharacterClass != CharClassPaladin)
		return;

	// Get Aura Flash Interval
	sprintf(_Key, "%s", "Aura Flash Interval");
	retval = server->GetHackProfileString("auto", _Character, _Key);

	if (strlen(retval))
	{
		AuraInterval = atoi(retval);
		//delete retval;
	}
	else
	{
		AuraInterval = 0;
		sprintf(_Value, "%d", 0);
		server->SetHackProfileString("auto", _Character, _Key, _Value);
	}

	// Setup Aura Flash Queue
    for (i=0; i<NumAura; i++)
    {
		sprintf(_Key, "Aura Flash %d", i+1);
		retval = server->GetHackProfileString("auto", _Character, _Key);
		if (strlen(retval))
		{
            AuraQueue[i] = atoi(retval);
            //delete retval;
        }
		else
		{
			sprintf(_Value, "%d", 0);
			server->SetHackProfileString("auto", _Character, _Key, _Value);
		}
	}
}

//////////////////////////////////////////////////////////////////////
// Something wrong with this routine!!!
//////////////////////////////////////////////////////////////////////
BOOL IsInTown()
{
	if (td.thisgame->player->PlayerLocation == 0x05)
		return TRUE;
	else
		return FALSE;
}

//////////////////////////////////////////////////////////////////////
// Display overhead message
//////////////////////////////////////////////////////////////////////
void OverheadMsg(char *Msg)
{
	BYTE aPacket[512];

	sprintf((char*)aPacket, "%c%c%c%s%c%c%c", 0x14, 0x00, 0x00, Msg, 0,0,0);
	server->GameSendPacketToServer(aPacket, strlen(Msg)+6);
}

//////////////////////////////////////////////////////////////////////
// Write Log File
// Log record format:
// 12/31/01 09:00:00 [<-10char->] Message
//////////////////////////////////////////////////////////////////////
void WriteLog(char *Tag, char *Msg)
{
	char DateBuffer[9], TimeBuffer[9];

	if (LogFile)
	{
		if (logfile = fopen("c:\\auto.log", "a+"))
		{
			_strdate(DateBuffer);
			_strtime(TimeBuffer);
			fprintf(logfile, "%s %s [%.10s] %s\n", DateBuffer, TimeBuffer, Tag, Msg);
			fclose(logfile);
			Tag[0] = Msg[0] = 0;
		}
		else
		{
			LogFile = 0;
			server->GamePrintError("Auto: Debug log disabled. Failed to open c:\\auto.log");
		}
	}
}

BOOL LogInPacket(BYTE Packet)
{
	BYTE InList[4] = {
		0x12, 0x23, 0x59, 0x94
	};

	for (int i=0; i<4; i++)
		if (Packet == InList[i])
			return TRUE;

	return FALSE;
}

BOOL LogOutPacket(BYTE Packet)
{
	BYTE OutList[7] = {
		0x0c, 0x08, 0x09, 0x0F, 0x10, 0x3c, 0x66
	};

	for (int i=0; i<7; i++)
		if (Packet == OutList[i])
			return TRUE;

	return FALSE;
}

// 
// stuff below here is from ClientCore.cpp
//////////////////////////////////////////////////////////////////////
// Dll entry/exit
//////////////////////////////////////////////////////////////////////
BOOL APIENTRY DllMain(HANDLE hModule, DWORD  ul_reason_for_call, LPVOID lpReserved)
{
	BOOL hResult = TRUE;
	char *t;
    switch (ul_reason_for_call)
	{
		case DLL_PROCESS_ATTACH:
			
			// Create server struct			
			server = new FUNCTIONENTRYPOINTS; 

			// Bind exported functions from server
			HMODULE hModule;
			hModule = (HMODULE)GetModuleHandle("D2HackIt");
			
			// Macros make this look a lot nicer :)
			BIND_TO_SERVER(GamePrintInfo);
			BIND_TO_SERVER(GamePrintError);
			BIND_TO_SERVER(GamePrintVerbose);
			BIND_TO_SERVER(GamePrintString);
			BIND_TO_SERVER(GameCommandLine);
			BIND_TO_SERVER(GameSendPacketToServer);
			BIND_TO_SERVER(GameSendPacketToGame);
			BIND_TO_SERVER(GetFingerprint);
			BIND_TO_SERVER(Intercept);
			BIND_TO_SERVER(GetHackProfileString);
			BIND_TO_SERVER(SetHackProfileString);
			BIND_TO_SERVER(GetThisgameStruct);

			// Get plugin path
			t = ConfigPath;
			if (GetModuleFileName((HINSTANCE)hModule, t, _MAX_PATH)) {
				int p=strlen(ConfigPath);
				while (p) {
						if (ConfigPath[p] == '\\')
							{ ConfigPath[p] = 0; p=0;}
						else
					p--;
				}
			}	 
			// initiate client
			// hResult = OnClientStart();
			break;

		case DLL_PROCESS_DETACH:
			// kill client

			// hResult = OnClientStop();

			delete server;
			break;
    } 
    return hResult;
}

//////////////////////////////////////////////////////////////////////
// Stubfunctions for 'property get' functions.
//////////////////////////////////////////////////////////////////////
LPCSTR	EXPORT GetModuleAuthor()		{return ModuleAuthor;}
LPCSTR	EXPORT GetModuleWebsite()		{return ModuleWebsite;}
DWORD	EXPORT GetModuleVersion()		{return ModuleVersion;}
LPCSTR	EXPORT GetModuleEmail()			{return ModuleEmail;}
LPCSTR	EXPORT GetModuleDescription()	{return ModuleDescription;}

//////////////////////////////////////////////////////////////////////
// OnClientCommandLine
// -------------------------------------------------------------------
// The modules own extension of the command line interface. Any custom
// commands you add are parsed here.
//
// Return value should be TRUE, but it is not used at this 
// time.
//
// Arguments when we get here:
// argv[0]			Name of module
// argv[1]			Name of command (If supplied)
// argv[2 ... n]	The rest
//
// Syntax in the game: .<module> <arguments>
//////////////////////////////////////////////////////////////////////
BOOL EXPORT OnGameCommandLine(char* argv[], int argc)
{
	// Check if user supplied anything at all, if not assume help...
	if (argc==1)
		argv[argc++]="help";


	MODULECOMMANDSTRUCT* mcs=ModuleCommands;

	while (mcs->szName) {
		if (!stricmp(mcs->szName, argv[1]))
			break;
		mcs++;
	}

	char *p,*t,*msg,*fMsg;
	fMsg=new char[256];
	//
	// Is this a built-in function ?
	if (mcs->szName) {
		//
		// If functions returns false, show usage help
		if (!mcs->pFunc(argv, argc)) {
			t=new char[strlen(mcs->szUsage)+1];
			server->GamePrintInfo("Usage:");
			sprintf((char*)t, "%s", mcs->szUsage);
			if (strlen((char*)t))
			{
				msg=p=t;
				while (*p != 0) {
					if (*p == '\n') 
					{
						*(p++) = 0;
						sprintf(fMsg, "c4.%s %s", argv[0], msg);
						server->GamePrintInfo((char*)fMsg);
					if (*p != 0)
						msg = p;
					} else
						p++;
					}
				sprintf(fMsg, "c4.%s %s", argv[0], msg);
				server->GamePrintInfo((char*)fMsg);
			}
			delete t;
		}
	} else {
	// Unknown command, show catch-all help phraze.
	t=new char[128];
	sprintf(t, "Unknown command c4'%s %s'c0 - try c4'.%s help'c0 to get help.",
		argv[0], argv[1], argv[0]);
	server->GamePrintError(t);
	delete t;
	}
	delete fMsg;
	return TRUE;
}


//////////////////////////////////////////////////////////////////////
// OnGameCommandHelp
// -------------------------------------------------------------------
// Our default help function.
//
// Syntax in the game: .<module> <arguments>
//////////////////////////////////////////////////////////////////////
BOOL PRIVATE OnGameCommandHelp(char** argv, int argc)
{
	// If command line is longer than 2, show usage for 'help'
	if (argc>2) return FALSE;

	char t[1024];
	sprintf(t, "Available commands for %s:", argv[0]);
	server->GamePrintInfo(t);

	// Loop through ModuleCommands[] and print their names
	for (int i=0; ModuleCommands[i].szName != NULL; i++)
	{
		sprintf(t, "c4.%s %s", argv[0], ModuleCommands[i].szName);
		server->GamePrintInfo(t);
	}

	sprintf(t, "For help on a specific command, type c4.%s <command> help", argv[0]);
	server->GamePrintInfo(t);
	return TRUE;
}
